from sqlalchemy.orm import Bundle
from database import db, ITEMS_PER_PAGE


class WhoImport(db.Model):
    __tablename__ = 'application__import__who'

    id = db.Column(db.Integer, primary_key=True)
    new_cases = db.Column(db.String(255), nullable=False)
    cumulative_cases = db.Column(db.String(255), nullable=False)
    new_deaths = db.Column(db.String(255), nullable=False)
    cumulative_deaths = db.Column(db.String(255), nullable=False)
    country_code = db.Column(db.String(255), nullable=False)
    country = db.Column(db.String(255), nullable=False)
    who_region = db.Column(db.String(255), nullable=False)
    date_reported = db.Column(db.String(255), nullable=False)
    datum = db.Column(db.Date, nullable=False)
    year = db.Column(db.Integer, nullable=False)
    month = db.Column(db.Integer, nullable=False)
    day_of_month = db.Column(db.Integer, nullable=False)
    day_of_week = db.Column(db.Integer, nullable=False)
    week_of_year = db.Column(db.Integer, nullable=False)
    year_week = db.Column(db.String(255), nullable=False)

    @classmethod
    def remove_all(cls):
        for one in cls.get_all():
            db.session.delete(one)
        db.session.commit()
        return None

    @classmethod
    def get_all_as_page(cls, page):
        return db.session.query(cls).order_by(
            cls.date_reported.desc(),
            cls.country.asc()
        ).paginate(page, per_page=ITEMS_PER_PAGE)

    @classmethod
    def get_all(cls):
        return db.session.query(cls).order_by(
            cls.date_reported.desc(),
            cls.country.asc()
        ).all()

    @classmethod
    def get_by_id(cls, other_id):
        return db.session.query(cls)\
            .filter(cls.id == other_id)\
            .one()

    @classmethod
    def get_regions(cls):
        return db.session.query(cls.who_region)\
            .order_by(cls.who_region)\
            .distinct().all()

    @classmethod
    def get_dates_reported(cls):
        return db.session.query(cls.date_reported)\
            .order_by(cls.date_reported.desc())\
            .distinct().all()

    @classmethod
    def get_for_one_day(cls, day):
        return db.session.query(cls)\
            .filter(cls.date_reported == day)\
            .order_by(cls.country.asc())\
            .all()

    @classmethod
    def get_dates_reported_as_array(cls):
        myresultarray = []
        myresultset = db.session.query(cls.date_reported)\
            .order_by(cls.date_reported.desc())\
            .group_by(cls.date_reported)\
            .distinct().all()
        for item, in myresultset:
            if not item in myresultarray:
                myresultarray.append(item)
        return myresultarray

    @classmethod
    def __get_new_dates_as_array_sql(cls):
        # TODO: #83 WhoImport.get_new_dates_as_array() SQLalchemy instead of SQL
        sql_query = """
            select
                distinct 
                    application__import__who.date_reported
                from
                    application__import__who
                where
                    date_reported
                not in (
                    select
                        distinct
                            who_datereported.date_reported_import_str
                        from
                            who
                        left join
                            who_datereported
                        on
                            who.date_reported_id=who_datereported.id
                        group by 
                            who_datereported.date_reported_import_str
                        order by
                            who_datereported.date_reported_import_str desc
                )
                group by
                    application__import__who.date_reported
                order by 
                    application__import__who.date_reported desc
            """
        new_dates = []
        for item in db.session.execute(sql_query):
            new_dates.append(item['date_reported'])
        return new_dates

    @classmethod
    def __get_new_dates_as_array_orm(cls):
        # TODO: #83 WhoImport.get_new_dates_as_array() SQLalchemy instead of SQL
        sql_query = """
                select
                    distinct 
                        application__import__who.date_reported
                    from
                        application__import__who
                    where
                        date_reported
                    not in (
                        select
                            distinct
                                who_datereported.date_reported_import_str
                            from
                                who
                            left join
                                who_datereported
                            on
                                who.date_reported_id=who_datereported.id
                            group by 
                                who_datereported.date_reported_import_str
                            order by
                                who_datereported.date_reported_import_str desc
                    )
                    group by
                        application__import__who.date_reported
                    order by 
                        application__import__who.date_reported desc
                """
        new_dates = []
        for item in db.session.execute(sql_query):
            new_dates.append(item['date_reported'])
        return new_dates

    @classmethod
    def get_new_dates_as_array(cls):
        #return cls.__get_new_dates_as_array_sql()
        return cls.__get_new_dates_as_array_orm()

    @classmethod
    def countries(cls):
        bu = Bundle('countries', cls.country_code, cls.country, cls.who_region)
        return db.session.query(bu).distinct()

    @classmethod
    def get_datum_of_all_who_import(cls):
        dates_reported = []
        bu = Bundle('dates_reported', cls.date_reported)
        for date_reported in db.session.query(bu).distinct().order_by(cls.date_reported):
            item = date_reported[0][0]
            if not item in dates_reported:
                dates_reported.append(item)
        return dates_reported


class WhoFlat(db.Model):
    __tablename__ = 'application__import__who_flat'

    id = db.Column(db.Integer, primary_key=True)
    date_reported_import_str = db.Column(db.String(255), nullable=False, unique=True)
    year_week = db.Column(db.String(255), nullable=False)
    datum = db.Column(db.Date, nullable=False, unique=True)
    year = db.Column(db.Integer, nullable=False)
    month = db.Column(db.Integer, nullable=False)
    day_of_month = db.Column(db.Integer, nullable=False)
    day_of_week = db.Column(db.Integer, nullable=False)
    week_of_year = db.Column(db.Integer, nullable=False)
    day_of_year = db.Column(db.Integer, nullable=True)
    year_day_of_year = db.Column(db.String(255), nullable=True, unique=True)
    location = db.Column(db.String(255), nullable=False)
    location_group = db.Column(db.String(255), nullable=False)
    new_cases = db.Column(db.String(255), nullable=False)
    cumulative_cases = db.Column(db.String(255), nullable=False)
    new_deaths = db.Column(db.String(255), nullable=False)
    cumulative_deaths = db.Column(db.String(255), nullable=False)
    country_code = db.Column(db.String(255), nullable=False)
    country = db.Column(db.String(255), nullable=False)
    who_region = db.Column(db.String(255), nullable=False)
    date_reported = db.Column(db.String(255), nullable=False)
